rm(list=ls())

library(data.table)
library(xts)
library(fBasics)
library(fAssets)

### read definitions of series
defs=fread("data/defs.txt",dec=".",sep="\t")
defs

### read prices and convert them to time series
prices=fread("data/prices.txt",dec=".",sep="\t")
prices=xts(prices[,-1],order.by=as.Date(prices$date))
plot(prices$Esp500)
prices

### calculate returns
returns=diff(prices,lag=1,log=TRUE,na.pad=FALSE)

nrow(prices)
nrow(returns)
head(prices[,1:5])
head(returns[,1:5])

plot(returns$Esp500)  
white=rnorm(nrow(returns))
white=xts(white,index(returns))

par(mfrow=c(1,2))
plot(returns$Esp500); plot(white) # note stationarity, volatilityt clustering
par(mfrow=c(1,1))


### descriptive stats
summary(returns)
summarytools::descr(returns,transpose=TRUE,round.digits=5)

# histogram: leptokurtosis, skew
histPlot(timeSeries(returns$Esp500)) # hist(returns$Esp500,breaks=60,freq=FALSE,col="lightgreen"); curve(dnorm(x, mean=mean(returns$Esp500), sd=sd(returns$Esp500)), add=TRUE, col="darkblue", lwd=2); lines(density(returns$Esp500), col="red", lwd=2); legend("topright",c("data","normal","kernel"),col=c("lightgreen","darkblue","red"),lwd=2)

# QQ-plot
qqnormPlot(timeSeries(returns$Esp500))

#### compare statitical properties 
boxPlot(coredata(returns[]),las=2)

plot(acf(returns$Esp500))
plot(pacf(returns$Esp500))

#### correlations
library(corrplot)
corr=cor(returns)
corrplot(corr,method="color",type="upper")
corrplot(corr,method="color",type="upper",
                   p.mat=cor.mtest(returns,conf.level=0.99)$p,sig.level=0.01)

