rm(list=ls())

library(data.table)
library(xts)
library(quadprog)
library(viridis)

### read data 
defs=fread("data/defs.txt",dec=".",sep="\t") # definitions of series
prices=fread("data/prices.txt",dec=".",sep="\t") # read prices 
prices=xts(prices[,-1],order.by=as.Date(prices$date)) # convert them to time series
returns=diff(prices,lag=1,log=TRUE,na.pad=FALSE) # calculate returns

### read data 
returns=returns[,c("Esp500","Eftse","Epacific","BtreasLong","BtreasShort","RealEstate")]
corrplot::corrplot(cor(returns),method="color",type="upper",
                   order="hclust",addCoef.col ="black",
                   diag=F)

n=ncol(returns)
n
histRets=colMeans(returns)
histRets
covm=cov(returns)
covm

#### min variance portfolio
#### min(-d^T x + 1/2 x^T D x)
#### A^T x >= b
dvec=matrix(c(0,0,0,0,0,0),nrow=n,ncol=1)  # dvec=matrix(0,nrow=n,ncol=1)
Dmat=covm
Amat=matrix(c(1,1,1,1,1,1),nrow=n,ncol=1)  # Amat=matrix(1,nrow=n,ncol=1)
bvec=1
sol=solve.QP(Dmat,dvec,Amat,bvec,meq=1)
model1=sol$solution
model1

#### min variance portfolio, long positions only
#### min(-d^T x + 1/2 x^T D x)
#### A^T x >= b
dvec=matrix(0,nrow=n,ncol=1)
dvec
Dmat=covm

Amat=matrix(1,nrow=n,ncol=1)  # Amat=matrix(1,nrow=n,ncol=1)
t(Amat)
Amat=cbind(Amat,diag(1,n))
t(Amat)

bvec=c(1,rep(0,n))
bvec
sol=solve.QP(Dmat,dvec,Amat,bvec,meq=1)
model2=sol$solution
model2

#### min variance portfolio, long only, above average expected return
#### min(-d^T x + 1/2 x^T D x)
#### A^T x >= b
dvec=matrix(0,nrow=n,ncol=1)
dvec
Dmat=covm

Amat=matrix(1,nrow=n,ncol=1)  # Amat=matrix(1,nrow=n,ncol=1)
Amat=cbind(Amat,diag(1,n))
Amat=cbind(Amat,histRets)
t(Amat)

bvec=c(1,rep(0,n),mean(histRets))
bvec
sol=solve.QP(Dmat,dvec,Amat,bvec,meq=1)
model3=sol$solution
model3

### compare the weights
weights=cbind(model1,model2,model3)
par(mar=c(3, 3, 1, 8))
barplot(weights,
        legend=names(returns),args.legend = list(x = "topright", bty = "n", inset=c(-0.45, 0)))
barplot(weights,col=viridis(6),
        legend=names(returns),args.legend = list(x = "topright", bty = "n", inset=c(-0.45, 0)))
par(mar=c(5.1, 4.1, 4.1, 2.1))

