rm(list=ls())

library(xts)

### read data 
prices=fread("data/prices.txt",dec=".",sep="\t") # read prices 
prices=xts(prices[,-1],order.by=as.Date(prices$date)) # convert them to time series
returns=diff(prices,lag=1,log=TRUE,na.pad=FALSE) # calculate returns

### Partial correlation function
partial.cor <- function (X, ...) 
{
  R <- cor(X, ...)
  RI <- solve(R)
  D <- 1/sqrt(diag(RI))
  R <- -RI * (D %o% D)
  diag(R) <- 0
  rownames(R) <- colnames(R) <- colnames(X)
  R
}

parcorr <- partial.cor(returns)

### create bootstrap sample
R <- 100 #number of replications
boots<- list()
for (i in 1:R){
  boots[[i]] <- as.data.frame(returns[sample(1:nrow(returns),nrow(returns), replace=TRUE),])
}

### see bootstrap responses 
#change index [1,2] if you want a different one.
boot.partial <- sapply(boots, function(x) partial.cor(x)[2,3])
hist(boot.partial)
### see confidence intervals for bootstrap responses
quantile(boot.partial, probs=c(.05, .95))
### compare to original value
parcorr[2,3]
