rm(list=ls())
library(quantspec)
library(zoo)
library(corrplot)
library(tseries)
library(quantilogram)

########################
#### Correlations
########################
data=read.csv(file="data.csv",header = T, dec = ".", sep = ";")
data$Date=as.Date(data$Date,format="%d.%m.%Y")
data=read.zoo(data[,-c(1)])

corrplot.mixed(cor(data), upper = "ellipse", lower = "number",lower.col = "black", number.cex = .7, number.digits = 2)


par(mfrow=c(1,2))
corrplot.mixed(cor(data), upper = "ellipse", lower = "number",lower.col = "black", number.cex = .7, number.digits = 2)
corrplot.mixed(cor(data,method="spearman"), upper = "ellipse", lower = "number",lower.col = "black", number.cex = .7, number.digits = 2)
par(mfrow=c(1,1))

########################
####DMCA
########################
DMCA_LRCC<-function(x,y,k){
  xx<-cumsum(x)
  yy<-cumsum(y)
  ma<-c(rep(1,k))/k
  X_MA<-filter(xx,ma)
  Y_MA<-filter(yy,ma)
  F2_xy<-mean((xx-X_MA)[(1+floor(k/2)):(length(xx)-floor(k/2))]*(yy-Y_MA)[(1+floor(k/2)):(length(yy)-floor(k/2))])
  F2_xx<-mean((xx-X_MA)[(1+floor(k/2)):(length(xx)-floor(k/2))]*(xx-X_MA)[(1+floor(k/2)):(length(xx)-floor(k/2))])
  F2_yy<-mean((yy-Y_MA)[(1+floor(k/2)):(length(yy)-floor(k/2))]*(yy-Y_MA)[(1+floor(k/2)):(length(yy)-floor(k/2))])
  rho<-F2_xy/sqrt(F2_xx*F2_yy)
  return(rho)
}

Z=data.frame(EUR,JPY,GBP,CHF,CAD,CNY,BTC,ETH,XRP,LTC,XLM,XEM)
names(Z)
matrix.A=matrix(0,nrow=length(Z),ncol=length(Z))
rownames(matrix.A)=names(Z)
colnames(matrix.A)=names(Z)

k=2 #for example
for (x in 1:length(Z)) {
  for (y in 1:length(Z)) {
    DMCA=DMCA_LRCC(Z[,x],Z[,y],k)
    matrix.A[x,y]=DMCA
  }
}

corrplot.mixed(matrix.A, upper = "ellipse", lower = "number",lower.col = "black", number.cex = .7, number.digits = 2)

########################
####Cross-Quantilogram
########################
k=2 #for example
q1=0.05
q2=0.05
cqs=matrix(0,ncol=ncol(data),nrow=ncol(data))
for (x in 1:ncol(data)) {
  for (y in 1:ncol(data)) {
    cq=CrossQ.StatBoot.OptAve(DATA = data[,c(x,y)],vecA=c(q1,q2),k=k,Bsize=10,sigLev=0.05)
    if (cq$vCRQ<cq$vecCV[1]) cqs[x,y]=cq$vCRQ
    if (cq$vCRQ>cq$vecCV[2]) cqs[x,y]=cq$vCRQ
  }
}
corrplot.mixed(cqs, upper = "ellipse", lower = "number",lower.col = "black", number.cex = .7, number.digits = 2)

########################
####Quantile coherency
########################
# data=read.csv(file="data.csv",header = T, dec = ".", sep = ";")
# data$Date=as.Date(data$Date,format="%d.%m.%Y")
# data=read.zoo(data[,-c(1)])

qqs <- c(0.05,0.5,0.95) #different quantiles
ffq=c(1/2,1/22,1/250)   #different frequencies
w = kernelWeight(W = W1, b = 0.5*dim(data)[1]^(-1/4))
sPG <- smoothedPG(data, levels.1=qqs, weight=w)
Coh <- getCoherency(sPG, frequencies = 2*pi*ffq)
CI <- getPointwiseCIs(sPG, quantity = "coherency",frequencies = 2*pi*ffq)

res=NULL; resl=NULL; resu=NULL;
for (frq in 1:3) {
  for (tau1 in 1:3) {
    for (tau2 in 1:3) {
      res=rbind(res,c(paste0("Freq:",1/ffq[frq],", q1:",qqs[tau1],", q2:",qqs[tau2]),rep("",11)))
      res=rbind(res,Re(Coh[frq,,tau1,,tau2,1]))
      
      resl=rbind(resl,c(paste0("Freq:",1/ffq[frq],", q1:",qqs[tau1],", q2:",qqs[tau2]),rep("",11)))
      resl=rbind(resl,Re(CI$lowerCIs[frq,,tau1,,tau2]))
      
      resu=rbind(resu,c(paste0("Freq:",1/ffq[frq],", q1:",qqs[tau1],", q2:",qqs[tau2]),rep("",11)))
      resu=rbind(resu,Re(CI$upperCIs[frq,,tau1,,tau2]))
    }
  }
}

########################
#visualization
#change here for different frequencies and quantiles to
#frq = c(1,2,3), tau1 = c(1,2,3), tau2 = c(1,2,3)

matrix=Re(Coh[frq=1,,tau1=1,,tau2=1,1])
rownames(matrix)=names(data);colnames(matrix)=names(data)
corrplot.mixed(matrix, upper = "ellipse", lower = "number",lower.col = "black", number.cex = .7, number.digits = 2)
