rm(list=ls())

library(data.table)
library(xts)

### read data 
defs=fread("data/defs.txt",dec=".",sep="\t") # definitions of series
prices=fread("data/prices.txt",dec=".",sep="\t") # read prices 
prices=xts(prices[,-1],order.by=as.Date(prices$date)) # convert them to time series
returns=diff(prices,lag=1,log=TRUE,na.pad=FALSE) # calculate returns
returns=timeSeries(returns)
allreturns=returns

#### Black-Litterman
library(BLCOP)
returns=allreturns[,c("Esp500","Eftse","Epacific","BtreasLong","BtreasShort","RealEstate")]
pickMatrix=matrix(c(1,0,0,0,0,0,
                    0,0,0,1,0,0),nrow=2,ncol=6)
views=BLViews(P=pickMatrix,q=c(-0.2,0.1),confidences=c(100,100),assetNames = names(returns))
views
priorMeans=colMeans(returns)
priorCov=cov(returns)
poster=posteriorEst(views=views,sigma=priorCov,mu=priorMeans,tau=1/2)
poster

optPort=optimalPortfolios.fPort(poster,optimizer="tangencyPortfolio")
optPort

par(mfcol=c(1,2))
weightsPie(optPort$priorOptimPortfolio)
weightsPie(optPort$posteriorOptimPortfolio)
par(mfrow=c(1,1))
densityPlots(poster,assetsSel = "Esp500")

### neural network autoregression
library(forecast)
series=prices$Esp500[2000:2326]
plot(series)
model=nnetar(series)
autoplot(forecast(model,h=30,PI=FALSE))

library(nnfor)
model=mlp(ts(series))
plot(model)
frc=forecast(model,h=50)
plot(frc)

library(neuralnet)
library(dplyr)
scale01 <- function(x){   (x - min(x)) / (max(x) - min(x)) }
series= data.frame(prices) %>% select(Esp500,BtreasLong,Eftse,Eemerging) %>% mutate_all(scale01)

train=series[1:2200,]
test =series[2201:2326,]

nn <- neuralnet(Esp500~BtreasLong+Eftse+Eemerging,data=train, hidden=c(2,1), linear.output=FALSE, threshold=0.01)
nn$result.matrix
plot(nn)
nnres=predict(nn,test[,c("BtreasLong","Eftse","Eemerging")])
results <- data.frame(actual = test$Esp500, prediction = nnres)
results
accuracy=1-abs(mean((results$actual-results$prediction)/results$actual))
accuracy

nn <- neuralnet(Esp500~BtreasLong+Eftse+Eemerging,data=train, hidden=c(5,2), linear.output=FALSE, threshold=0.01)
nn$result.matrix
plot(nn)
nnres=predict(nn,test[,c("BtreasLong","Eftse","Eemerging")])
results <- data.frame(actual = test$Esp500, prediction = nnres)
results
accuracy=1-abs(mean((results$actual-results$prediction)/results$actual))
accuracy
